@@ -74,6 +74,16 @@ Q_LOGGING_CATEGORY(LOG_JXRPLUGIN, "kf.imageformats.plugins.jxr", QtWarningMsg)
 
 // #define JXR_ENABLE_ADVANCED_METADATA
 
+/* *** JXR_MAX_IMAGE_WIDTH and JXR_MAX_IMAGE_HEIGHT ***
+ * The maximum size in pixel allowed by the plugin.
+ */
+#ifndef JXR_MAX_IMAGE_WIDTH
+#define JXR_MAX_IMAGE_WIDTH KIF_LARGE_IMAGE_PIXEL_LIMIT
+#endif
+#ifndef JXR_MAX_IMAGE_HEIGHT
+#define JXR_MAX_IMAGE_HEIGHT JXR_MAX_IMAGE_WIDTH
+#endif
+
 #ifndef JXR_MAX_METADATA_SIZE
 /*!
  * XMP and EXIF maximum size.
@@ -386,6 +396,11 @@ public:
         if (pDecoder) {
             qint32 w, h;
             pDecoder->GetSize(pDecoder, &w, &h);
+            if (w > JXR_MAX_IMAGE_WIDTH || h > JXR_MAX_IMAGE_HEIGHT || w < 1 || h < 1) {
+                qCCritical(LOG_JXRPLUGIN) << "JXRHandlerPrivate::imageSize() Maximum image size is limited to" << JXR_MAX_IMAGE_WIDTH << "x"
+                                          << JXR_MAX_IMAGE_HEIGHT << "pixels";
+                return {};
+            }
             return QSize(w, h);
         }
         return {};
@@ -564,9 +579,7 @@ public:
         // IMPORTANT: these values must be in exactMatchingFormat()
         // clang-format off
         auto valid = QSet<QImage::Format>()
-#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
             << QImage::Format_CMYK8888
-#endif
 #ifndef JXR_DENY_FLOAT_IMAGE
             << QImage::Format_RGBA16FPx4
             << QImage::Format_RGBX16FPx4
@@ -656,10 +669,8 @@ public:
         wmiSCP->bVerbose = FALSE;
         if (fmt == QImage::Format_Grayscale8 || fmt == QImage::Format_Grayscale16 || fmt == QImage::Format_Mono)
             wmiSCP->cfColorFormat = Y_ONLY;
-#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
         else if (fmt == QImage::Format_CMYK8888)
             wmiSCP->cfColorFormat = CMYK;
-#endif
         else
             wmiSCP->cfColorFormat = YUV_444;
         wmiSCP->bdBitDepth = BD_LONG;
@@ -797,10 +808,8 @@ private:
             << std::pair<QImage::Format, PKPixelFormatGUID>(QImage::Format_RGBA32FPx4_Premultiplied, GUID_PKPixelFormat128bppPRGBAFloat)
             << std::pair<QImage::Format, PKPixelFormatGUID>(QImage::Format_RGBX32FPx4, GUID_PKPixelFormat128bppRGBFloat)
 #endif // JXR_DENY_FLOAT_IMAGE
-#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
             << std::pair<QImage::Format, PKPixelFormatGUID>(QImage::Format_CMYK8888, GUID_PKPixelFormat32bppCMYK)
             << std::pair<QImage::Format, PKPixelFormatGUID>(QImage::Format_CMYK8888, GUID_PKPixelFormat32bppCMYKDIRECT)
-#endif
             << std::pair<QImage::Format, PKPixelFormatGUID>(QImage::Format_Mono, GUID_PKPixelFormatBlackWhite)
             << std::pair<QImage::Format, PKPixelFormatGUID>(QImage::Format_Grayscale8, GUID_PKPixelFormat8bppGray)
             << std::pair<QImage::Format, PKPixelFormatGUID>(QImage::Format_Grayscale16, GUID_PKPixelFormat16bppGray)
@@ -962,8 +971,14 @@ bool JXRHandler::read(QImage *outImage)
     if (auto err = d->pDecoder->GetResolution(d->pDecoder, &hres, &vres)) {
         qCWarning(LOG_JXRPLUGIN) << "JXRHandler::read() error while reading resolution:" << err;
     } else {
-        img.setDotsPerMeterX(qRound(hres * 1000 / 25.4));
-        img.setDotsPerMeterY(qRound(vres * 1000 / 25.4));
+        const qint32 hdpm = dpi2ppm(hres);
+        if (hdpm > 0) {
+            img.setDotsPerMeterX(hdpm);
+        }
+        const qint32 vdpm = dpi2ppm(vres);
+        if (vdpm > 0) {
+            img.setDotsPerMeterY(vdpm);
+        }
     }
 
     // alpha copy mode
@@ -1053,6 +1068,10 @@ bool JXRHandler::write(const QImage &image)
         qCWarning(LOG_JXRPLUGIN) << "JXRHandler::write() image too large: the image cannot exceed 4GB.";
         return false;
     }
+    if (image.width() > JXR_MAX_IMAGE_WIDTH || image.height() > JXR_MAX_IMAGE_HEIGHT) {
+        qCCritical(LOG_JXRPLUGIN) << "JXRHandler::write() Maximum image size is limited to" << JXR_MAX_IMAGE_WIDTH << "x" << JXR_MAX_IMAGE_HEIGHT << "pixels";
+        return {};
+    }
 
     if (!d->initForWriting()) {
         return false;
@@ -1105,7 +1124,7 @@ bool JXRHandler::write(const QImage &image)
         qCWarning(LOG_JXRPLUGIN) << "JXRHandler::write() error while setting the image size:" << err;
         return false;
     }
-    if (auto err = d->pEncoder->SetResolution(d->pEncoder, qi.dotsPerMeterX() * 25.4 / 1000, qi.dotsPerMeterY() * 25.4 / 1000)) {
+    if (auto err = d->pEncoder->SetResolution(d->pEncoder, dppm2dpi(qi.dotsPerMeterX()), dppm2dpi(qi.dotsPerMeterY()))) {
         qCWarning(LOG_JXRPLUGIN) << "JXRHandler::write() error while setting the image resolution:" << err;
         return false;
     }
