@@ -33,14 +33,20 @@
 #include "scanlineconverter_p.h"
 #include "util_p.h"
 
-#include <QDataStream>
-#include <QDebug>
-#include <QImage>
 #include <QColorSpace>
+#include <QDataStream>
+#include <QImage>
+#include <QLoggingCategory>
 
 #include <cmath>
 #include <cstring>
 
+#ifdef QT_DEBUG
+Q_LOGGING_CATEGORY(LOG_PSDPLUGIN, "kf.imageformats.plugins.psd", QtDebugMsg)
+#else
+Q_LOGGING_CATEGORY(LOG_PSDPLUGIN, "kf.imageformats.plugins.psd", QtWarningMsg)
+#endif
+
 typedef quint32 uint;
 typedef quint16 ushort;
 typedef quint8 uchar;
@@ -72,10 +78,20 @@ typedef quint8 uchar;
  */
 // #define PSD_FORCE_RGBA
 
+/* *** PSD_MAX_IMAGE_WIDTH and PSD_MAX_IMAGE_HEIGHT ***
+ * The maximum size in pixel allowed by the plugin.
+ */
+#ifndef PSD_MAX_IMAGE_WIDTH
+#define PSD_MAX_IMAGE_WIDTH KIF_LARGE_IMAGE_PIXEL_LIMIT
+#endif
+#ifndef PSD_MAX_IMAGE_HEIGHT
+#define PSD_MAX_IMAGE_HEIGHT PSD_MAX_IMAGE_WIDTH
+#endif
+
 namespace // Private.
 {
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 8, 0) || defined(PSD_NATIVE_CMYK_SUPPORT_DISABLED)
+#if defined(PSD_NATIVE_CMYK_SUPPORT_DISABLED)
 #   define CMYK_FORMAT QImage::Format_Invalid
 #else
 #   define CMYK_FORMAT QImage::Format_CMYK8888
@@ -311,11 +327,18 @@ static PSDImageResourceSection readImageResourceSection(QDataStream &s, bool *ok
     *ok = true;
 
     // Section size
-    qint32 sectioSize;
-    s >> sectioSize;
+    quint32 tmpSize;
+    s >> tmpSize;
+    qint64 sectioSize = tmpSize;
 
     // Reading Image resource block
     for (auto size = sectioSize; size > 0;) {
+
+#define DEC_SIZE(value) \
+        if ((size -= qint64(value)) < 0) { \
+            *ok = false; \
+            break; }
+
         // Length      Description
         // -------------------------------------------------------------------
         // 4           Signature: '8BIM'
@@ -330,10 +353,10 @@ static PSDImageResourceSection readImageResourceSection(QDataStream &s, bool *ok
 
         quint32 signature;
         s >> signature;
-        size -= sizeof(signature);
+        DEC_SIZE(sizeof(signature))
         // NOTE: MeSa signature is not documented but found in some old PSD take from Photoshop 7.0 CD.
         if (signature != S_8BIM && signature != S_MeSa) { // 8BIM and MeSa
-            qDebug() << "Invalid Image Resource Block Signature!";
+            qCDebug(LOG_PSDPLUGIN) << "Invalid Image Resource Block Signature!";
             *ok = false;
             break;
         }
@@ -341,7 +364,7 @@ static PSDImageResourceSection readImageResourceSection(QDataStream &s, bool *ok
         // id
         quint16 id;
         s >> id;
-        size -= sizeof(id);
+        DEC_SIZE(sizeof(id))
 
         // getting data
         PSDImageResourceBlock irb;
@@ -349,33 +372,41 @@ static PSDImageResourceSection readImageResourceSection(QDataStream &s, bool *ok
         // name
         qint32 bytes = 0;
         irb.name = readPascalString(s, 2, &bytes);
-        size -= bytes;
+        DEC_SIZE(bytes)
 
         // data read
         quint32 dataSize;
         s >> dataSize;
-        size -= sizeof(dataSize);
-        // NOTE: Qt device::read() and QDataStream::readRawData() could read less data than specified.
-        //       The read code should be improved.
-        if (auto dev = s.device())
-            irb.data = dev->read(dataSize);
+        DEC_SIZE(sizeof(dataSize))
+        if (auto dev = s.device()) {
+            if (dataSize > size) {
+                qCDebug(LOG_PSDPLUGIN) << "Invalid Image Resource Block Data Size!";
+                *ok = false;
+                break;
+            }
+            irb.data = deviceRead(dev, dataSize);
+        }
         auto read = irb.data.size();
-        if (read > 0)
-            size -= read;
-        if (quint32(read) != dataSize) {
-            qDebug() << "Image Resource Block Read Error!";
+        if (read > 0) {
+            DEC_SIZE(read)
+        }
+        if (read != qint64(dataSize)) {
+            qCDebug(LOG_PSDPLUGIN) << "Image Resource Block Read Error!";
             *ok = false;
             break;
         }
 
         if (auto pad = dataSize % 2) {
             auto skipped = s.skipRawData(pad);
-            if (skipped > 0)
-                size -= skipped;
+            if (skipped > 0) {
+                DEC_SIZE(skipped);
+            }
         }
 
         // insert IRB
         irs.insert(id, irb);
+
+#undef DEC_SIZE
     }
 
     return irs;
@@ -473,7 +504,13 @@ PSDColorModeDataSection readColorModeDataSection(QDataStream &s, bool *ok = null
 
     qint32 size;
     s >> size;
-    if (size != 768) {  // read the duotone data (524 bytes)
+    if (size < 0) {
+        *ok = false;
+    } else if (size > 8 * 1024 * 1024) {
+        // The known color sections are all in the order of a few hundred bytes.
+        // I skip the ones that are too big, I don't know what to do with them.
+        *ok = s.skipRawData(size) == size;
+    } else if (size != 768) {  // read the duotone data (524 bytes)
         // NOTE: A RGB/Gray float image has a 112 bytes ColorModeData that could be
         //       the "32-bit Toning Options" of Photoshop (starts with 'hdrt').
         //       Official Adobe specification tells "Only indexed color and duotone
@@ -586,17 +623,21 @@ static bool setResolution(QImage &img, const PSDImageResourceSection &irs)
     s >> i32;                               // Horizontal resolution in pixels per inch.
     if (i32 <= 0)
         return false;
-    auto hres = fixedPointToDouble(i32);
+    auto hres = dpi2ppm(fixedPointToDouble(i32));
 
     s.skipRawData(4);                       // Display data (not used here)
 
     s >> i32;                               // Vertial resolution in pixels per inch.
     if (i32 <= 0)
         return false;
-    auto vres = fixedPointToDouble(i32);
+    auto vres = dpi2ppm(fixedPointToDouble(i32));
 
-    img.setDotsPerMeterX(hres * 1000 / 25.4);
-    img.setDotsPerMeterY(vres * 1000 / 25.4);
+    if (hres > 0) {
+        img.setDotsPerMeterX(hres);
+    }
+    if (vres > 0) {
+        img.setDotsPerMeterY(vres);
+    }
     return true;
 }
 
@@ -647,18 +688,18 @@ static QDataStream &operator>>(QDataStream &s, PSDHeader &header)
 static bool IsValid(const PSDHeader &header)
 {
     if (header.signature != 0x38425053) { // '8BPS'
-        // qDebug() << "PSD header: invalid signature" << header.signature;
+        // qCDebug(LOG_PSDPLUGIN) << "PSD header: invalid signature" << header.signature;
         return false;
     }
     if (header.version != 1 && header.version != 2) {
-        qDebug() << "PSD header: invalid version" << header.version;
+        qCDebug(LOG_PSDPLUGIN) << "PSD header: invalid version" << header.version;
         return false;
     }
     if (header.depth != 8 &&
         header.depth != 16 &&
         header.depth != 32 &&
         header.depth != 1) {
-        qDebug() << "PSD header: invalid depth" << header.depth;
+        qCDebug(LOG_PSDPLUGIN) << "PSD header: invalid depth" << header.depth;
         return false;
     }
     if (header.color_mode != CM_RGB &&
@@ -669,17 +710,17 @@ static bool IsValid(const PSDHeader &header)
         header.color_mode != CM_LABCOLOR &&
         header.color_mode != CM_MULTICHANNEL &&
         header.color_mode != CM_BITMAP) {
-        qDebug() << "PSD header: invalid color mode" << header.color_mode;
+        qCDebug(LOG_PSDPLUGIN) << "PSD header: invalid color mode" << header.color_mode;
         return false;
     }
     // Specs tells: "Supported range is 1 to 56" but when the alpha channel is present the limit is 57:
     // Photoshop does not make you add more (see also 53alphas.psd test case).
     if (header.channel_count < 1 || header.channel_count > 57) {
-        qDebug() << "PSD header: invalid number of channels" << header.channel_count;
+        qCDebug(LOG_PSDPLUGIN) << "PSD header: invalid number of channels" << header.channel_count;
         return false;
     }
-    if (header.width > 300000 || header.height > 300000) {
-        qDebug() << "PSD header: invalid image size" << header.width << "x" << header.height;
+    if (header.width > uint(std::min(300000, PSD_MAX_IMAGE_WIDTH)) || header.height > uint(std::min(300000, PSD_MAX_IMAGE_HEIGHT))) {
+        qCDebug(LOG_PSDPLUGIN) << "PSD header: invalid image size" << header.width << "x" << header.height;
         return false;
     }
     return true;
@@ -963,7 +1004,7 @@ inline void cmykToRgb(uchar *target, qint32 targetChannels, const char *source,
     auto invmax = 1.0 / max; // speed improvements by ~10%
 
     if (sourceChannels < 2) {
-        qDebug() << "cmykToRgb: image is not a valid MCH/CMYK!";
+        qCDebug(LOG_PSDPLUGIN) << "cmykToRgb: image is not a valid MCH/CMYK!";
         return;
     }
 
@@ -1012,7 +1053,7 @@ inline void labToRgb(uchar *target, qint32 targetChannels, const char *source, q
     auto invmax = 1.0 / max;
 
     if (sourceChannels < 3) {
-        qDebug() << "labToRgb: image is not a valid LAB!";
+        qCDebug(LOG_PSDPLUGIN) << "labToRgb: image is not a valid LAB!";
         return;
     }
 
@@ -1153,39 +1194,39 @@ public:
 
         // Check image file format.
         if (stream.atEnd() || !IsValid(m_header)) {
-            // qDebug() << "This PSD file is not valid.";
+            // qCDebug(LOG_PSDPLUGIN) << "This PSD file is not valid.";
             return false;
         }
 
         // Check if it's a supported format.
         if (!IsSupported(m_header)) {
-            // qDebug() << "This PSD file is not supported.";
+            // qCDebug(LOG_PSDPLUGIN) << "This PSD file is not supported.";
             return false;
         }
 
         // Color Mode Data section
         m_cmds = readColorModeDataSection(stream, &ok);
         if (!ok) {
-            qDebug() << "Error while skipping Color Mode Data section";
+            qCDebug(LOG_PSDPLUGIN) << "Error while skipping Color Mode Data section";
             return false;
         }
 
         // Image Resources Section
         m_irs = readImageResourceSection(stream, &ok);
         if (!ok) {
-            qDebug() << "Error while reading Image Resources Section";
+            qCDebug(LOG_PSDPLUGIN) << "Error while reading Image Resources Section";
             return false;
         }
         // Checking for merged image (Photoshop compatibility data)
         if (!hasMergedData()) {
-            qDebug() << "No merged data found";
+            qCDebug(LOG_PSDPLUGIN) << "No merged data found";
             return false;
         }
 
         // Layer and Mask section
         m_lms = readLayerAndMaskSection(stream, isPsb(), &ok);
         if (!ok) {
-            qDebug() << "Error while skipping Layer and Mask section";
+            qCDebug(LOG_PSDPLUGIN) << "Error while skipping Layer and Mask section";
             return false;
         }
 
@@ -1246,19 +1287,19 @@ bool PSDHandler::read(QImage *image)
     quint16 compression;
     stream >> compression;
     if (compression > 1) {
-        qDebug() << "Unknown compression type";
+        qCDebug(LOG_PSDPLUGIN) << "Unknown compression type";
         return false;
     }
 
     const QImage::Format format = d->format();
     if (format == QImage::Format_Invalid) {
-        qWarning() << "Unsupported image format. color_mode:" << header.color_mode << "depth:" << header.depth << "channel_count:" << header.channel_count;
+        qCWarning(LOG_PSDPLUGIN) << "Unsupported image format. color_mode:" << header.color_mode << "depth:" << header.depth << "channel_count:" << header.channel_count;
         return false;
     }
 
     img = imageAlloc(d->size(), format);
     if (img.isNull()) {
-        qWarning() << "Failed to allocate image, invalid dimensions?" << QSize(header.width, header.height);
+        qCWarning(LOG_PSDPLUGIN) << "Failed to allocate image, invalid dimensions?" << QSize(header.width, header.height);
         return false;
     }
     img.fill(qRgb(0, 0, 0));
@@ -1273,7 +1314,7 @@ bool PSDHandler::read(QImage *image)
     auto native_cmyk = img.format() == CMYK_FORMAT;
 
     if (header.height > kMaxQVectorSize / header.channel_count / sizeof(quint32)) {
-        qWarning() << "LoadPSD() header height/channel_count too big" << header.height << header.channel_count;
+        qCWarning(LOG_PSDPLUGIN) << "LoadPSD() header height/channel_count too big" << header.height << header.channel_count;
         return false;
     }
 
@@ -1316,7 +1357,7 @@ bool PSDHandler::read(QImage *image)
         // CMYK with spots (e.g. CMYKA) ICC conversion to RGBA/RGBX
         QImage tmpCmyk;
         ScanLineConverter iccConv(img.format());
-#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0) && !defined(PSD_NATIVE_CMYK_SUPPORT_DISABLED)
+#if !defined(PSD_NATIVE_CMYK_SUPPORT_DISABLED)
         if (header.color_mode == CM_CMYK && img.format() != QImage::Format_CMYK8888) {
             auto tmpi = QImage(header.width, 1, QImage::Format_CMYK8888);
             if (setColorSpace(tmpi, irs))
@@ -1332,12 +1373,12 @@ bool PSDHandler::read(QImage *image)
             for (qint32 c = 0; c < header.channel_count; ++c) {
                 auto strideNumber = c * qsizetype(h) + y;
                 if (!device->seek(stridePositions.at(strideNumber))) {
-                    qDebug() << "Error while seeking the stream of channel" << c << "line" << y;
+                    qCDebug(LOG_PSDPLUGIN) << "Error while seeking the stream of channel" << c << "line" << y;
                     return false;
                 }
                 auto &&strideSize = strides.at(strideNumber);
                 if (!readChannel(rawStride, stream, strideSize, compression)) {
-                    qDebug() << "Error while reading the stream of channel" << c << "line" << y;
+                    qCDebug(LOG_PSDPLUGIN) << "Error while reading the stream of channel" << c << "line" << y;
                     return false;
                 }
 
@@ -1418,7 +1459,7 @@ bool PSDHandler::read(QImage *image)
             for (qint32 y = 0, h = header.height; y < h; ++y) {
                 auto&& strideSize = strides.at(c * qsizetype(h) + y);
                 if (!readChannel(rawStride, stream, strideSize, compression)) {
-                    qDebug() << "Error while reading the stream of channel" << c << "line" << y;
+                    qCDebug(LOG_PSDPLUGIN) << "Error while reading the stream of channel" << c << "line" << y;
                     return false;
                 }
 
@@ -1451,7 +1492,7 @@ bool PSDHandler::read(QImage *image)
 
     // Resolution info
     if (!setResolution(img, irs)) {
-        // qDebug() << "No resolution info found!";
+        // qCDebug(LOG_PSDPLUGIN) << "No resolution info found!";
     }
 
     // ICC profile
@@ -1468,23 +1509,21 @@ bool PSDHandler::read(QImage *image)
         if (header.color_mode == CM_RGB && header.depth == 32) {
             img.setColorSpace(QColorSpace(QColorSpace::SRgbLinear));
         }
-#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
         if (header.color_mode == CM_GRAYSCALE && header.depth == 32) {
             auto qs = QColorSpace(QPointF(0.3127, 0.3291), QColorSpace::TransferFunction::Linear);
             qs.setDescription(QStringLiteral("Linear grayscale"));
             img.setColorSpace(qs);
         }
-#endif
     }
 
     // XMP data
     if (!setXmpData(img, irs)) {
-        // qDebug() << "No XMP data found!";
+        // qCDebug(LOG_PSDPLUGIN) << "No XMP data found!";
     }
 
     // EXIF data
     if (!setExifData(img, d->m_exif)) {
-        // qDebug() << "No EXIF data found!";
+        // qCDebug(LOG_PSDPLUGIN) << "No EXIF data found!";
     }
 
     // Duotone images: color data contains the duotone specification (not documented).
@@ -1555,7 +1594,7 @@ QVariant PSDHandler::option(ImageOption option) const
 bool PSDHandler::canRead(QIODevice *device)
 {
     if (!device) {
-        qWarning("PSDHandler::canRead() called with no device");
+        qCWarning(LOG_PSDPLUGIN) << "PSDHandler::canRead() called with no device";
         return false;
     }
 
